<?php

/**
 * Library for admin settings
 *
 * @package    WordPress
 * @author     David Perez <david@closemarketing.es>
 * @copyright  2019 Closemarketing
 * @version    1.0
 */
defined( 'ABSPATH' ) || exit;
/**
 * Library for WooCommerce Settings
 *
 * Settings in order to sync products
 *
 * @package    WordPress
 * @author     David Perez <david@closemarketing.es>
 * @copyright  2019 Closemarketing
 * @version    0.1
 */
class WCIMPH_Admin
{
    /**
     * The plugin file
     *
     * @var string
     */
    private  $file ;
    /**
     * Construct and intialize
     *
     * @param string $file File of this class.
     */
    public function __construct( $file )
    {
        $this->file = $file;
        // * Adds Settings options
        add_filter( 'woocommerce_get_sections_products', array( $this, 'wc_add_section' ) );
        add_filter(
            'woocommerce_get_settings_products',
            array( $this, 'wc_all_settings' ),
            10,
            2
        );
        add_action( 'admin_menu', function () {
            add_submenu_page(
                'woocommerce',
                __( 'Import from Holded', 'import-holded-products-woocommerce' ),
                __( 'Import Holded Products', 'import-holded-products-woocommerce' ),
                'manage_options',
                'admin.php?page=wc-settings&tab=products&section=wcpimh-holded'
            );
        }, 101 );
        add_action( 'admin_head', array( $this, 'custom_css' ) );
    }
    
    /**
     * Create the section beneath the products tab
     **/
    /**
     * Create section beneath the products tabs
     *
     * @param  array $sections Sections of WooCommerce Settings.
     * @return array $sections
     */
    public function wc_add_section( $sections )
    {
        $sections['wcpimh-holded'] = __( 'Import products from Holded', 'import-holded-products-woocommerce' );
        return $sections;
    }
    
    /**
     * Add settings to the specific section we created before
     *
     * @param array $settings Settings of WooCommerce.
     * @param array $current_section Actual section of WooCommerce.
     * @return $settings
     */
    public function wc_all_settings( $settings, $current_section )
    {
        /**
         * Check the current section is what we want
         */
        
        if ( 'wcpimh-holded' === $current_section ) {
            $settings_wcpimh = array();
            // Add Title to the Settings.
            $settings_wcpimh[] = array(
                'name' => __( 'Holded Product Importing Settings', 'import-holded-products-woocommerce' ),
                'type' => 'title',
                'desc' => sprintf( __( 'Put the connection API key settings in order to connect and sync products. You can go here <a href="%s" target="_blank">App Holded API</a>', 'import-holded-products-woocommerce' ), 'https://app.holded.com/api' ),
                'id'   => 'wcpimh',
            );
            // Add second text field option.
            $settings_wcpimh[] = array(
                'name'     => __( 'Holded API Key', 'import-holded-products-woocommerce' ),
                'desc_tip' => '',
                'id'       => 'wcpimh_api',
                'type'     => 'text',
            );
            // Final Prices.
            $settings_wcpimh[] = array(
                'name'     => __( 'Regular price with tax included?', 'import-holded-products-woocommerce' ),
                'desc_tip' => '',
                'id'       => 'wcpimh_taxinc',
                'type'     => 'select',
                'options'  => [
                'yes' => __( 'Include Regular price with tax', 'import-holded-products-woocommerce' ),
                'no'  => __( 'Not include Regular price with tax', 'import-holded-products-woocommerce' ),
            ],
            );
            // Sync Stock?.
            $settings_wcpimh[] = array(
                'name'     => __( 'Import stock?', 'import-holded-products-woocommerce' ),
                'desc_tip' => '',
                'id'       => 'wcpimh_stock',
                'type'     => 'select',
                'options'  => [
                'yes' => __( 'Yes', 'import-holded-products-woocommerce' ),
                'no'  => __( 'No', 'import-holded-products-woocommerce' ),
            ],
            );
            // Default status?.
            $settings_wcpimh[] = array(
                'name'     => __( 'Default status for new products?', 'import-holded-products-woocommerce' ),
                'desc_tip' => '',
                'id'       => 'wcpimh_prodst',
                'type'     => 'select',
                'options'  => [
                'draft'   => __( 'Draft', 'import-holded-products-woocommerce' ),
                'publish' => __( 'Publish', 'import-holded-products-woocommerce' ),
                'pending' => __( 'Pending', 'import-holded-products-woocommerce' ),
                'private' => __( 'Private', 'import-holded-products-woocommerce' ),
            ],
            );
            // Category Import?.
            $label_cat = __( 'Category separator', 'import-holded-products-woocommerce' );
            if ( cmk_fs()->is_not_paying() ) {
                $label_cat .= __( '(ONLY PREMIUM VERSION)', 'import-holded-products-woocommerce' );
            }
            $settings_wcpimh[] = array(
                'name'     => $label_cat,
                'desc_tip' => 'Use the separator for product categories',
                'id'       => 'wcpimh_catsep',
                'type'     => 'text',
            );
            $settings_wcpimh[] = array(
                'type' => 'sectionend',
                'id'   => 'wcpimh',
            );
            return $settings_wcpimh;
            /**
             * If not, return the standard settings
             */
        } else {
            return $settings;
        }
    
    }
    
    /**
     * Custom CSS for admin
     *
     * @return void
     */
    public function custom_css()
    {
        // Free Version.
        echo  '
			<style>
			.wp-admin .woocommerce span.wcpimh-premium{ 
				color: #b4b9be;
			}
			.wp-admin .woocommerce #wcpimh_catnp,
			.wp-admin .woocommerce #wcpimh_stock,
			.wp-admin .woocommerce #wcpimh_catsep {
				width: 70px;
			}
			.wp-admin .woocommerce #wcpimh_prodst {
				width: 150px;
			}
			.wp-admin .woocommerce #wcpimh_api,
			.wp-admin .woocommerce #wcpimh_taxinc {
				width: 270px;
			}' ;
        // Not premium version.
        if ( cmk_fs()->is_not_paying() ) {
            echo  'input[name=wcpimh_catsep] {
				pointer-events:none;
			}' ;
        }
        echo  '</style>' ;
    }

}
new WCIMPH_Admin( __FILE__ );